#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    interface_main.py
#       
#    Copyright 2010 Victor Ramirez <virako.9@gmail.com>
#       
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import gtk
import gobject
import threading
import time
from opencv.highgui import cvCreateFileCapture, cvQueryFrame, cvSaveImage
from opencv.cv import cvCloneImage, CV_GRAY2RGB

import create_sgf
from function import convert_opencv2gtk, convert_opencv_gray2gtk, load_image_no_webcam
from interface_configuration import Interface_configuration
from interface_about import Interface_about
from camera import Camera
from video import Video
from cte import WITH_VIDEO, WITH_WEBCAM, THRESHOLD
from intersection import Intersection
from difference import difference
from luminosity import search_circle

gtk.gdk.threads_init()

class Interface_main():
    """Clase utilizada para las interfaz princpipal. """
    def __init__(self):
        
        # variables
        self.values = []
        self.capture = None
        self.file_sgf = None
        self.photos = []
        self.count = 0
        self.configuration_open = False
        
        # cargamos el archivo creado con glade
        self.gladefile = "gui/main.glade"
        self.glade = gtk.Builder()
        self.glade.add_from_file(self.gladefile)

        #cargamos las variables
        self.window = self.glade.get_object("window_main")
        self.image_left = self.glade.get_object("main_image_left")
        self.image_right = self.glade.get_object("main_image_right")
        self.button_search = self.glade.get_object("button_search_intersection")
        self.button_ok = self.glade.get_object("button_intersection_ok")
        self.button_foto = self.glade.get_object("button_foto")
        self.button_end = self.glade.get_object("button_end")
        self.menu_file_new = self.glade.get_object("menu_file_new")
        self.progressbar = self.glade.get_object("progressbar")
        
        # conectamos las señales
        self.glade.connect_signals(self)
        
        # modificamos las variables para mostrar o no los botones
        self.window.set_icon_from_file("images/icono.png")
        self.window.set_title("Tablerogo")
        self.button_foto.set_sensitive(False)
        self.button_ok.set_sensitive(False)
        self.button_search.set_sensitive(False)
        self.button_end.set_sensitive(False)
        
        # mostramos la ventana
        self.window.show()
        
        # abrimos nueva configuracion al iniciar 
        self.menu_file_new_activate(self.menu_file_new)

        
    # callback 
    def menu_file_new_activate(self, widget):
        """Callback que es llamado cuando se pulsa el botón de Nuevo en la 
         barra de menú de la interfaz y que nos abre una nueva interfaz 
         de configuración. """
        
        # creamos un objeto camera
        self.camera = Camera()
        # Comprobamos que está abierta una ventana, si no abrimos una nueva 
        if not self.configuration_open: 
            self.i_configuration = Interface_configuration(self.camera)
            self.configuration_open = True
            # creamos bucle para comprobar cuando creamos una nueva configuracion
            self.checking = gobject.timeout_add(500, self.check_configuration)        
        else: # si esta abierta le pasamos el foco # TODO
            self.i_configuration.window.activate_default()
                
    def menu_help_about_activate(self, widget):
        """Callback que es llamado cuando se pulsa el botón de Acerca de en la 
         barra de menú de la interfaz y que nos abre una nueva ventana. """
        Interface_about()
    
    def menu_language_1_activate(self, widget):
        """Callback que es llamado cuando se pulsa el botón del idioma 1 en la 
         barra de menú de la interfaz y que nos cambia al idioma seleccionado. 
         """
        print "cambiado a lenguaje_1"
        
    def menu_language_2_activate(self, widget):
        """Callback que es llamado cuando se pulsa el botón del idioma 2 en la 
         barra de menú de la interfaz y que nos cambia al idioma seleccionado. 
         """
        print "cambiado a lenguaje_2"
    
    def menu_file_exit_activate(self, widget):
        """Callback que es llamado cuando pulsamos la opción de salir de la 
         barra menú. """
        gtk.main_quit()
        exit()
        
        
    def button_end_clicked(self, widget):
        """Callback que es llamado cuando pulsamos el botón terminar"""
        self.end_file_sgf()        
        
    def button_search_intersection_clicked(self, widget):
        """Callback que es llamado para buscar las intersecciones del tablero 
         cuando pulsamos el boton Buscar Intersecciones. """
        
        gobject.source_remove(self.refresh)
        self.quit = False
        self.new_thread(self.thread_progressbar)
        self.new_thread(self.thread_search_intersection)
        self.progressbar.show()
        
    def button_intersection_ok_clicked(self, widget):
        """Callback que es llamado cuando pulsamos el boton Intersecciones OK, 
         lo que hace es activar el foton de foto y el de terminar para que 
         empecemos a buscar piedras. """
        self.button_foto.set_sensitive(True)
        self.button_search.set_sensitive(False)
        self.button_ok.set_sensitive(False)
        self.button_end.set_sensitive(True)
        self.image_right.hide()
        widget.hide()
        self.window.reshow_with_initial_size()
        
    def button_foto_clicked(self, widget):
        """Callback que es llamado cuando pulsamos el boton Foto, el cual nos  
         busca los cambios producidos en el tablero. """
        
        save = cvCloneImage(self.frame)
        self.photos.append(save)
        ima_diff = difference(self.photos[self.count], self.photos[self.count-1])
        
        try: cvSaveImage("images/%s.png" %self.count, ima_diff)
        except: print "Error al guardar la diferencia " # para chequeo

        stones = search_circle(ima_diff, self.intersection.tablero, \
         self.intersection.size)
        values = self.intersection.search_intersection(stones)
        print "Valores: ", values
        # Insertar valor de la piedra puesta en un label a mano derecha
        # Insertar el color de la piedra puesta en una imagen a la derecha
        self.file_sgf.add_text(values[0], values[1]) # guardamos jugada en sgf
        if values[2]: # terminar 
            self.end_file_sgf()
        self.count += 1


    def gtk_main_quit(self, widget):
        """Funcion que es llamada cuando queremos cerramos la ventana. """
        gtk.main_quit()
        exit()
    
    
    # metodos 
    def load_image_left(self, image):
        """Nos carga una imagen en la parte izquierda de la pantalla. 
        @param image: imagen a cargar.
        @type image: L{IplImage}"""
        pix_buf = convert_opencv2gtk(image)
        self.image_left.set_from_pixbuf(pix_buf)
    
    def load_image_right(self, image):
        """Nos carga una imagen en la parte derecha de la pantalla. 
        @param image: imagen a cargar.
        @type image: L{IplImage}"""
        pix_buf = convert_opencv_gray2gtk(image)
        self.image_right.set_from_pixbuf(pix_buf)
    
    def check_configuration(self):
        """Comprueba si la ventana de configuración está abierta. 
        @return: True si la configuración se ha creado correctamente, 
         False en caso contrario.
        @rtype: C{bool}"""
        if not self.i_configuration.open_window_configuration:
            self.configuration_open = False
            if self.i_configuration.configuration_ok:
                self.configuration_applicate()
                return False
        return True
        
    def end_file_sgf(self):
        """Función que cierra el archivo sgf y desactiva los botones hasta 
         nueva configuración. """
        self.file_sgf.end_file()
        self.button_foto.set_sensitive(False)
        self.button_search.set_sensitive(False)
        self.button_ok.set_sensitive(False)
        self.button_end.set_sensitive(False)
        self.image_left.hide()
        
        # terminamos con el refresh
        gobject.source_remove(self.refresh)
        
        self.configuration_initial()
        # que muestre que tu archivo se ha creado correctamente
        
    
    def configuration_initial(self):
        """Funcion que vuelve a la configuración inicial. """
        self.window.reshow_with_initial_size()
        self.values = []
        self.capture = None
        self.file_sgf = None
        self.photos = []
        self.count = 0
        self.configuration_open = False
    
        
    def configuration_applicate(self):
        """Función que aplica la configuración creada en la interfaz 
         configuracion."""
        # obtenemos los valores que queremos guardar de configuracion
        self.values = self.i_configuration.values
        if WITH_VIDEO == len(self.values):
            # creamos un objeto video
            self.video = Video()
            self.video.open_video(self.values[1])
            frame = self.video.get_frame()
            self.sel = WITH_VIDEO
        elif WITH_WEBCAM == len(self.values):
            frame = self.camera.get_frame()
            self.sel = WITH_WEBCAM
        
        self.load_image_left(frame)
        # crear archivo sgf
        self.file_sgf = create_sgf.Create_sgf(self.values[0])
        self.refresh = gobject.timeout_add(50, self.update)
        self.button_search.set_sensitive(True)
        self.image_left.show()
        
        
    def update(self):
        """Función que actualiza el video, cámara o imagen. 
         @return: Devuelve True siempre.
         @rtype: C{bool}
         """
        if WITH_VIDEO == self.sel:
            self.frame = self.video.get_frame()
        elif WITH_WEBCAM == self.sel:
            self.frame = self.camera.get_frame()
        self.load_image_left(self.frame)
        return True
    
    
    def new_thread(self, method):
        """Funcion que crea un nuevo hilo. """
        t = threading.Thread(target=method, args=())
        t.start()
    
    
    def thread_progressbar(self):
        """ Función hilo que muestra la barra de progreso. """
        while not self.quit:
            time.sleep(0.01)
            gtk.gdk.threads_enter()
            self.progressbar.pulse()
            gtk.gdk.threads_leave() 
        
        
    def thread_search_intersection(self):    
        """ Función hilo que busca las intersecciones del tablero. """
        
        self.button_search.set_sensitive(False)
        save = cvCloneImage(self.frame)
        self.photos.append(save)
        self.intersection = Intersection(self.photos[self.count])
        # buscamos las intersecciones
        if self.intersection.detect_intersections(THRESHOLD) != -1:
            # Pintamos las intersecciones encontradas en la imagen de la derecha
            ima = self.intersection.media()
            self.load_image_right(ima)
            self.image_right.show()
            self.button_ok.show()
            # Activamos button_ok
            self.button_ok.set_sensitive(True)
        
        self.button_search.set_sensitive(True)
        self.count += 1
        
        self.quit = True # para parar la barra de progreso
        self.progressbar.hide()
        self.refresh = gobject.timeout_add(50, self.update)# Bucle video 
        
        
