#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    tablero.py
#       
#    Copyright 2010 Victor Ramirez <virako.9@gmail.com>
#       
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.



from cte import SIZE, SPACE

class Tablero:
    """Clase que nos crea una matriz tablero para poder guardar todos los datos 
       de las intersecciones y las piedras puestas"""

    def __init__(self, size = SIZE):  # size es el tamaño del tablero, 19 por defecto
        self.size = size
        # Esto nos crea una matriz de escalar 4-tuplas, las cuales usaremos para (x pos imagen, y pos imagen, ocupado, num_jugada)
        self.matriz  = [[[0,0,{0:SPACE}] for x in range(size)] for y in range(size)]
        self.square = [0,0]
    
    def get_x_y(self):
        """Funcion que nos devuelve en que coordenadas se encuentra la 
           intersección del tablero 1-1, y la 19-19"""
        x1 = self.matriz[0][0][0] - self.square[0]
        y1 = self.matriz[0][1][1] - self.square[1]
        x2 = self.matriz[self.size-1][self.size-1][0] + self.square[0]
        y2 = self.matriz[self.size-1][self.size-1][1] + self.square[1] 
        return x1, x2, y1, y2
    
    
    def get_square(self):
        """Función que cambia el valor de la variable self.square, la cual 
           contiene la información del tamaño de un cuadrado del tablero"""
        self.square[0] = self.matriz[1][0][0] - self.matriz[0][0][0]
        self.square[1] = self.matriz[0][1][1] - self.matriz[0][0][1]

    def __str__(self):
        cadena = ""
        for x in range(self.size):
            for y in range(self.size):
                cadena += str(self.matriz[x][y][2].items()[-1]) + "  " 
                if y == 18:
                    cadena += "\n"
        return cadena
    
    
